% FRC_TAPERMASK Generate a taper mask for FRC computation
%
% SYNOPSIS:
%   out = frc_tapermask(imagesize,x_domain,y_domain,zoomfactor)
%
% PARAMETERS:
%
%   imagesize
%      Size in CCD pixels of the original localization data
%   x_domain
%       x-coordinates of the mask corners
%   y_domain
%       y-coordinates of the mask corners
%   zoomfactor
%      Magnification of the mask image w.r.t. the original localization data
%
% DEFAULTS:
%   zoomfactor  = 1
%
%
% (C) Copyright 2013               Quantitative Imaging Group
%     All rights reserved          Faculty of Applied Physics
%                                  Delft University of Technology
%                                  Lorentzweg 1
%                                  2628 CJ Delft
%                                  The Netherlands
% Robert Nieuwenhuizen, Oct 2013

function out = frc_tapermask(sz,x_domain,y_domain,zm)

if numel(sz)>1
    sz = max(sz);
end

if isempty(zm)
    zm = 1;
end

if isempty(x_domain)
    x_domain = [1 sz];
end

if isempty(y_domain)
    y_domain = [1 sz];
end

if numel(x_domain) == 1
    x_domain = [1 x_domain];
end

if numel(y_domain) == 1
    y_domain = [1 y_domain];
end

x_domain = ceil(x_domain*zm);
y_domain = ceil(y_domain*zm);

nfac = 8;

x_im = xx(x_domain(2)-x_domain(1)+1,y_domain(2)-y_domain(1)+1)/(x_domain(2)-x_domain(1)+1);
mask = 0.5-0.5*cos(pi*nfac*x_im);          
mask(abs(x_im)<((nfac-2)/(nfac*2))) = 1;

y_im = yy(x_domain(2)-x_domain(1)+1,y_domain(2)-y_domain(1)+1)/(y_domain(2)-y_domain(1)+1);
mask_y = 0.5-0.5*cos(pi*nfac*y_im);          
mask_y(abs(y_im)<((nfac-2)/(nfac*2))) = 1;
mask = mask*mask_y;

out = newim(ceil(zm*sz),ceil(zm*sz));
out((x_domain(1):x_domain(2))-1,(y_domain(1):y_domain(2))-1) = mask;
