%MASKLOCALIZATIONS   Remove localizations that do not fall inside the mask
%
% SYNOPSIS:
%   [positions_kept positions_removed] = masklocalizations(positions,mask,zoomfactor)
%
% PARAMETERS:
%   zoomfactor
%      Pixels per unit distance in the positions list
%
% DEFAULTS:
%   zoomfactor  = 1
%
% NOTES:
%  Columns of positions are assumed to have the same dimensionality.
%  Localizations with x or y-values smaller than -0.5 are discarded.

% (C) Copyright 2012               Quantitative Imaging Group
%     All rights reserved          Faculty of Applied Physics
%                                  Delft University of Technology
%                                  Lorentzweg 1
%                                  2628 CJ Delft
%                                  The Netherlands
% Robert Nieuwenhuizen, Jan 2013


function varargout = masklocalizations(varargin)

d = struct('menu','FRC resolution',...
           'display','Mask localizations',...
           'inparams',struct('name',       {'positions',            'mask',             'zm'},...
                             'description',{'Localizations',        'Mask image',       'Zoom'},...
                             'type',       {'array',                'image',            'array'},...
                             'dim_check',  {{[],[-1 2],[-1 -1]},    2,                  {[],0}},...
                             'range_check',{'R',                    [],            	[eps Inf]},...
                             'required',   {1,                      1,                  0},...
                             'default',    {[],                     [],                 1}...
                              ),...
           'outparams',struct('name',{'positions_out','position_removed'},...
                              'description',{'Masked localizations','Removed localizations'},...
                              'type',{'array','array'}...
                              )...
           );       

if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      varargout{1} = d;
      return
   end
end

try
   [positions,mask,zm] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error('Parameter parsing was unsuccessful.')
   end
end       

if size(positions,2)>size(positions,1)
    warning('Unexpected input size.');
end

% Check that an output image size can be defined
if isempty(positions)
    out = [];
    return
end

if isempty(zm)
    zm = 1;
end

% Remove localizations that are not in the mask
s = ismember(floor(zm*(0.5+positions(:,1:2))),findcoord(mask),'rows');
varargout{1} = positions(s,:);
varargout{2} = positions(~s,:);


