%SELECTLOCALIZATIONS   Select localizations
%
% SYNOPSIS:
%   [out, roi, mask_image] = selectlocalizations(positions,zoomfactor)
%
% PARAMETERS:
%   zoomfactor
%      Pixels per unit distance in the positions list
%
% DEFAULTS:
%   zoomfactor  = 1
%
% NOTES:
%  Columns of positions are assumed to have the same dimensionality.
%  Localizations with x or y-values smaller than -0.5 are discarded.

% (C) Copyright 2012               Quantitative Imaging Group
%     All rights reserved          Faculty of Applied Physics
%                                  Delft University of Technology
%                                  Lorentzweg 1
%                                  2628 CJ Delft
%                                  The Netherlands
% Robert Nieuwenhuizen, Jan 2013


function varargout = selectlocalizations(varargin)

d = struct('menu','FRC resolution',...
           'display','Select localizations',...
           'inparams',struct('name',       {'positions',        'zm'},...
                             'description',{'Localizations',    'Zoom'},...
                             'type',       {'array',            'array'},...
                             'dim_check',  {{[],[-1 2],[-1 -1]},{[],0}},...
                             'range_check',{'R',                [eps Inf]},...
                             'required',   {1,                  0},...
                             'default',    {[],                 1}...
                              ),...
           'outparams',struct('name',{'positions_out','roi','mask_image'},...
                              'description',{'Masked localizations','ROI','mask_image'},...
                              'type',{'array','array','image'},...
                              'suppress',{1,1,1}...
                              )...
           );       

if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      varargout{1} = d;
      return
   end
end

try
   [positions,zm] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error('Parameter parsing was unsuccessful.')
   end
end       

if size(positions,2)>size(positions,1)
    warning('Unexpected input size.');
end

% Check that an output image size can be defined
if isempty(positions)
    varargout{1} = [];
    return
end

if isempty(zm)
    zm = 1;
end

% Ask for coordinates of bounding box
%im_locs = showlocalizations(positions,[],zm,'blobs',3);
im_locs = showlocalizations(positions,[],zm,'bin');
h1_example = dipshow(gaussf(im_locs,1),[0 0.5]);
diptruesize(h1_example,'max')
colormap('hot')
questdlg('Select a rectangular ROI by clicking and holding the left mouse button','Hint','OK','OK');
[tmp roi_sel] = dipcrop(h1_example);

% Create a mask of localizations that are to be removed
im_locs(:,:) = 0;
im_locs(roi_sel(1,1):roi_sel(1,1)+roi_sel(2,1),roi_sel(1,2):roi_sel(1,2)+roi_sel(2,2)) = 1;

close(h1_example)

% Remove localizations in selected region
varargout{1} = masklocalizations(positions,im_locs,zm);

roi_sel(2,:) = roi_sel(2,:) + 1;
roi_sel = roi_sel / zm;
roi_sel(1,:) = roi_sel(1,:) - 0.5;
varargout{2} = roi_sel;
varargout{3} = im_locs;
